from enum import Enum
from typing import Optional

from dbt.config.renderer import ProfileRenderer


class DataSourceType(Enum):
    SNOWFLAKE = "snowflake"
    REDSHIFT = "redshift"
    POSTGRES = "postgres"
    MSSQL = "mssql"


class Profile:
    def __init__(self, **config) -> None:
        del config["password"]
        self._config: dict[str, str] = ProfileRenderer().render_data(config)

    @classmethod
    def from_dict(cls, profile: dict, target: Optional[str] = None) -> "Profile":
        target = profile.get("target", target)
        if not target:
            raise ValueError("Target must be set")
        profile = profile.get("outputs", {}).get(target, {})
        return cls(**profile)

    @property
    def type(self) -> DataSourceType:
        return DataSourceType(self._config["type"])

    @property
    def host(self) -> str:
        return self._config["host"]

    @property
    def account(self) -> Optional[str]:
        return self._config.get("account")

    def get(self, key: str) -> Optional[str]:
        return self._config.get(key)



from enum import Enum
from typing import Optional, Type

from dbt.contracts.graph.nodes import ModelNode
from oddrn_generator import (
    Generator,
    MssqlGenerator,
    PostgresqlGenerator,
    RedshiftGenerator,
    SnowflakeGenerator,
)

from odd_dbt.models.profile import DataSourceType, Profile


class GeneratorAdapter:
    def __init__(self, generator_cls: Type[Generator], profile: Profile) -> None:
        self.generator_cls = generator_cls
        self.profile = profile

    def get_oddrn_for(self, model: ModelNode, path: Optional[str] = None) -> str:
        host = self.profile.get("host")
        database = self.profile.get("database") or self.profile.get("dbname")
        generator = self.generator_cls(host_settings=host, databases=database)

        name = model.name
        path = "views" if model.config.materialized == "view" else "tables"
        generator.set_oddrn_paths(**{"schemas": model.schema})
        return generator.get_oddrn_by_path(path, name)


class SnowflakeGeneratorAdapter(GeneratorAdapter):
    def get_oddrn_for(self, model: ModelNode, path: Optional[str] = None) -> str:
        host = f"{self.profile.get('account').upper()}.snowflakecomputing.com"
        database = (
            self.profile.get("database").upper() or self.profile.get("dbname").upper()
        )

        generator = self.generator_cls(host_settings=host, databases=database)
        name = model.name.upper()
        path = "views" if model.config.materialized == "view" else "tables"
        generator.set_oddrn_paths(**{"schemas": model.schema.upper()})
        return generator.get_oddrn_by_path(path, name)


def get_generator(profile: Profile) -> GeneratorAdapter:
    DATA_SOURCE_GENERATORS: dict[
        Enum, tuple[Type[GeneratorAdapter], Type[Generator], Profile]
    ] = {
        DataSourceType.POSTGRES: (GeneratorAdapter, PostgresqlGenerator, profile),
        DataSourceType.SNOWFLAKE: (
            SnowflakeGeneratorAdapter,
            SnowflakeGenerator,
            profile,
        ),
        DataSourceType.REDSHIFT: (GeneratorAdapter, RedshiftGenerator, profile),
        DataSourceType.MSSQL: (GeneratorAdapter, MssqlGenerator, profile),
    }

    if generator_config := DATA_SOURCE_GENERATORS.get(profile.type):
        return generator_config[0](generator_config[1], generator_config[2])
    else:
        raise ValueError(
            f"Unknown profile type {profile.type}. Available: {DATA_SOURCE_GENERATORS.keys()}"
        )
